@@ -31,7 +31,8 @@ void SkirtBrim::getFirstLayerOutline(SliceDataStorage& storage, const size_t pri
         constexpr bool include_support = false; //Include manually below.
         constexpr bool include_prime_tower = false; //Include manually below.
         constexpr bool external_outlines_only = false; //Remove manually below.
-        first_layer_outline = storage.getLayerOutlines(layer_nr, include_support, include_prime_tower, external_outlines_only);
+        constexpr bool for_brim = true;
+        first_layer_outline = storage.getLayerOutlines(layer_nr, include_support, include_prime_tower, external_outlines_only, for_brim);
         first_layer_outline = first_layer_outline.unionPolygons(); //To guard against overlapping outlines, which would produce holes according to the even-odd rule.
         Polygons first_layer_empty_holes;
         if (external_only)
@@ -67,7 +68,7 @@ void SkirtBrim::getFirstLayerOutline(SliceDataStorage& storage, const size_t pri
             first_layer_outline.add(support_layer.support_bottom);
             first_layer_outline.add(support_layer.support_roof);
         }
-        if (storage.primeTower.enabled)
+        if (storage.primeTower.enabled && !train.settings.get<bool>("prime_tower_brim_enable"))
         {
             first_layer_outline.add(storage.primeTower.outer_poly_first_layer); // don't remove parts of the prime tower, but make a brim for it
         }
@@ -83,7 +84,7 @@ void SkirtBrim::getFirstLayerOutline(SliceDataStorage& storage, const size_t pri
     }
 }
 
-coord_t SkirtBrim::generatePrimarySkirtBrimLines(const coord_t start_distance, size_t primary_line_count, const coord_t primary_extruder_minimal_length, const Polygons& first_layer_outline, Polygons& skirt_brim_primary_extruder)
+coord_t SkirtBrim::generatePrimarySkirtBrimLines(const coord_t start_distance, size_t& primary_line_count, const coord_t primary_extruder_minimal_length, const Polygons& first_layer_outline, Polygons& skirt_brim_primary_extruder)
 {
     const Settings& adhesion_settings = Application::getInstance().current_slice->scene.current_mesh_group->settings.get<ExtruderTrain&>("adhesion_extruder_nr").settings;
     const coord_t primary_extruder_skirt_brim_line_width = adhesion_settings.get<coord_t>("skirt_brim_line_width") * adhesion_settings.get<Ratio>("initial_layer_line_width_factor");
@@ -115,7 +116,7 @@ coord_t SkirtBrim::generatePrimarySkirtBrimLines(const coord_t start_distance, s
     return offset_distance;
 }
 
-void SkirtBrim::generate(SliceDataStorage& storage, Polygons first_layer_outline, const coord_t start_distance, const size_t primary_line_count, const bool allow_helpers /*= true*/)
+void SkirtBrim::generate(SliceDataStorage& storage, Polygons first_layer_outline, const coord_t start_distance, size_t primary_line_count, const bool allow_helpers /*= true*/)
 {
     const bool is_skirt = start_distance > 0;
     Scene& scene = Application::getInstance().current_slice->scene;
@@ -147,11 +148,16 @@ void SkirtBrim::generate(SliceDataStorage& storage, Polygons first_layer_outline
 
     coord_t offset_distance = generatePrimarySkirtBrimLines(gap, primary_line_count, primary_extruder_minimal_length, first_layer_outline, skirt_brim_primary_extruder);
 
+    // Skirt needs to be 'locked' first, otherwise the optimizer can change to order, which can cause undesirable outcomes w.r.t combo w. support-brim or prime-tower brim.
+    // If this method is called multiple times, the max order shouldn't reset to 0, so the maximum is taken.
+    storage.skirt_brim_max_locked_part_order[adhesion_extruder_nr] = std::max(is_skirt ? primary_line_count : 0, storage.skirt_brim_max_locked_part_order[adhesion_extruder_nr]);
+
     // handle support-brim
     const ExtruderTrain& support_infill_extruder = scene.current_mesh_group->settings.get<ExtruderTrain&>("support_infill_extruder_nr");
     if (allow_helpers && support_infill_extruder.settings.get<bool>("support_brim_enable"))
     {
-        generateSupportBrim(storage);
+        const bool merge_with_model_skirtbrim = !is_skirt;
+        generateSupportBrim(storage, merge_with_model_skirtbrim);
     }
 
     // generate brim for ooze shield and draft shield
@@ -225,7 +231,7 @@ void SkirtBrim::generate(SliceDataStorage& storage, Polygons first_layer_outline
     }
 }
 
-void SkirtBrim::generateSupportBrim(SliceDataStorage& storage)
+void SkirtBrim::generateSupportBrim(SliceDataStorage& storage, const bool merge_with_model_skirtbrim)
 {
     constexpr coord_t brim_area_minimum_hole_size_multiplier = 100;
 
@@ -286,11 +292,19 @@ void SkirtBrim::generateSupportBrim(SliceDataStorage& storage)
 
     if (support_brim.size())
     {
-        // to ensure that the skirt brim is printed from outside to inside, the support brim lines must
-        // come before the skirt brim lines in the Polygon object so that the outermost skirt brim line
-        // is at the back of the list
-        support_brim.add(skirt_brim);
-        skirt_brim = support_brim;
+        if (merge_with_model_skirtbrim)
+        {
+            // to ensure that the skirt brim is printed from outside to inside, the support brim lines must
+            // come before the skirt brim lines in the Polygon object so that the outermost skirt brim line
+            // is at the back of the list
+            support_brim.add(skirt_brim);
+            skirt_brim = support_brim;
+        }
+        else
+        {
+            // OTOH, if we use a skirt instead of a brim for the polygon, the skirt line(s) should _always_ come first.
+            skirt_brim.add(support_brim);
+        }
     }
 }
 
