@@ -1143,33 +1143,34 @@ void GCodeExport::writeFanCommand(double speed)
     {
         return;
     }
-    if (speed > 0)
+    if(flavor == EGCodeFlavor::MAKERBOT)
     {
-        if (flavor == EGCodeFlavor::MAKERBOT)
-            *output_stream << "M126 T0" << new_line; //value = speed * 255 / 100 // Makerbot cannot set fan speed...;
+        if(speed >= 50)
+        {
+            *output_stream << "M126 T0" << new_line; //Makerbot cannot PWM the fan speed...
+        }
         else
         {
-            *output_stream << "M106 S" << PrecisionedDouble{1, speed * 255 / 100};
-            if (fan_number)
-            {
-                *output_stream << " P" << fan_number;
-            }
-            *output_stream << new_line;
+            *output_stream << "M127 T0" << new_line;
         }
     }
+    else if (speed > 0)
+    {
+        *output_stream << "M106 S" << PrecisionedDouble{1, speed * 255 / 100};
+        if (fan_number)
+        {
+            *output_stream << " P" << fan_number;
+        }
+        *output_stream << new_line;
+    }
     else
     {
-        if (flavor == EGCodeFlavor::MAKERBOT)
-            *output_stream << "M127 T0" << new_line;
-        else
+        *output_stream << "M107";
+        if (fan_number)
         {
-            *output_stream << "M107";
-            if (fan_number)
-            {
-                *output_stream << " P" << fan_number;
-            }
-            *output_stream << new_line;
+            *output_stream << " P" << fan_number;
         }
+        *output_stream << new_line;
     }
 
     current_fan_speed = speed;
@@ -1177,11 +1178,35 @@ void GCodeExport::writeFanCommand(double speed)
 
 void GCodeExport::writeTemperatureCommand(const size_t extruder, const Temperature& temperature, const bool wait)
 {
-    if (!Application::getInstance().current_slice->scene.extruders[extruder].settings.get<bool>("machine_nozzle_temp_enabled"))
+    const ExtruderTrain& extruder_train = Application::getInstance().current_slice->scene.extruders[extruder];
+
+    if (!extruder_train.settings.get<bool>("machine_nozzle_temp_enabled"))
     {
         return;
     }
 
+    if (extruder_train.settings.get<bool>("machine_extruders_share_heater"))
+    {
+        // extruders share a single heater
+        if (extruder != current_extruder)
+        {
+            // ignore all changes to the non-current extruder
+            return;
+        }
+
+        // sync all extruders with the change to the current extruder
+        const size_t extruder_count = Application::getInstance().current_slice->scene.extruders.size();
+
+        for (size_t extruder_nr = 0; extruder_nr < extruder_count; extruder_nr++)
+        {
+            if (extruder_nr != extruder)
+            {
+                extruder_attr[extruder_nr].waited_for_temperature = wait;
+                extruder_attr[extruder_nr].currentTemperature = temperature;
+            }
+        }
+    }
+
     if ((!wait || extruder_attr[extruder].waited_for_temperature) && extruder_attr[extruder].currentTemperature == temperature)
     {
         return;
@@ -1359,7 +1384,7 @@ void GCodeExport::ResetLastEValueAfterWipe(size_t extruder)
 
 void GCodeExport::insertWipeScript(const WipeScriptConfig& wipe_config)
 {
-    Point3 prev_position = currentPosition;
+    const Point3 prev_position = currentPosition;
     writeComment("WIPE_SCRIPT_BEGIN");
 
     if (wipe_config.retraction_enable)
