@@ -40,11 +40,51 @@ void createLayerWithParts(const Settings& settings, SliceLayer& storageLayer, Sl
 
     std::vector<PolygonsPart> result;
     const bool union_layers = settings.get<bool>("meshfix_union_all");
-    result = layer->polygons.splitIntoParts(union_layers || union_all_remove_holes);
+    const ESurfaceMode surface_only = settings.get<ESurfaceMode>("magic_mesh_surface_mode");
+    if (surface_only == ESurfaceMode::SURFACE && !union_layers)
+    { // Don't do anything with overlapping areas; no union nor xor
+        result.reserve(layer->polygons.size());
+        for (const PolygonRef poly : layer->polygons)
+        {
+            result.emplace_back();
+            result.back().add(poly);
+        }
+    }
+    else
+    {
+        result = layer->polygons.splitIntoParts(union_layers || union_all_remove_holes);
+    }
+    const coord_t hole_offset = settings.get<coord_t>("hole_xy_offset");
     for(unsigned int i=0; i<result.size(); i++)
     {
         storageLayer.parts.emplace_back();
-        storageLayer.parts[i].outline = result[i];
+        if (hole_offset != 0)
+        {
+            // holes are to be expanded or shrunk
+            PolygonsPart outline;
+            Polygons holes;
+            for (const PolygonRef poly : result[i])
+            {
+                if (poly.orientation())
+                {
+                    outline.add(poly);
+                }
+                else
+                {
+                    holes.add(poly.offset(hole_offset));
+                }
+            }
+            for (PolygonRef hole : holes.unionPolygons().intersection(outline))
+            {
+                hole.reverse();
+                outline.add(hole);
+            }
+            storageLayer.parts[i].outline = outline;
+        }
+        else
+        {
+            storageLayer.parts[i].outline = result[i];
+        }
         storageLayer.parts[i].boundaryBox.calculate(storageLayer.parts[i].outline);
     }
 }
