@@ -1,4 +1,4 @@
-//Copyright (c) 2019 Ultimaker B.V.
+//Copyright (c) 2020 Ultimaker B.V.
 //CuraEngine is released under the terms of the AGPLv3 or higher.
 
 #include <stdio.h>
@@ -15,15 +15,16 @@
 #include "utils/SparsePointGridInclusive.h"
 
 
-namespace cura {
+namespace cura
+{
 
-int largest_neglected_gap_first_phase = MM2INT(0.01); //!< distance between two line segments regarded as connected
-int largest_neglected_gap_second_phase = MM2INT(0.02); //!< distance between two line segments regarded as connected
-int max_stitch1 = MM2INT(10.0); //!< maximal distance stitched between open polylines to form polygons
+constexpr int largest_neglected_gap_first_phase = MM2INT(0.01); //!< distance between two line segments regarded as connected
+constexpr int largest_neglected_gap_second_phase = MM2INT(0.02); //!< distance between two line segments regarded as connected
+constexpr int max_stitch1 = MM2INT(10.0); //!< maximal distance stitched between open polylines to form polygons
 
 void SlicerLayer::makeBasicPolygonLoops(Polygons& open_polylines)
 {
-    for(unsigned int start_segment_idx = 0; start_segment_idx < segments.size(); start_segment_idx++)
+    for(size_t start_segment_idx = 0; start_segment_idx < segments.size(); start_segment_idx++)
     {
         if (!segments[start_segment_idx].addedToPolygon)
         {
@@ -34,7 +35,7 @@ void SlicerLayer::makeBasicPolygonLoops(Polygons& open_polylines)
     segments.clear();
 }
 
-void SlicerLayer::makeBasicPolygonLoop(Polygons& open_polylines, unsigned int start_segment_idx)
+void SlicerLayer::makeBasicPolygonLoop(Polygons& open_polylines, const size_t start_segment_idx)
 {
 
     Polygon poly;
@@ -56,14 +57,14 @@ void SlicerLayer::makeBasicPolygonLoop(Polygons& open_polylines, unsigned int st
     open_polylines.add(poly);
 }
 
-int SlicerLayer::tryFaceNextSegmentIdx(const SlicerSegment& segment, int face_idx, unsigned int start_segment_idx) const
+int SlicerLayer::tryFaceNextSegmentIdx(const SlicerSegment& segment, const int face_idx, const size_t start_segment_idx) const
 {
     decltype(face_idx_to_segment_idx.begin()) it;
     auto it_end = face_idx_to_segment_idx.end();
     it = face_idx_to_segment_idx.find(face_idx);
     if (it != it_end)
     {
-        int segment_idx = (*it).second;
+        const int segment_idx = (*it).second;
         Point p1 = segments[segment_idx].start;
         Point diff = segment.end - p1;
         if (shorterThen(diff, largest_neglected_gap_first_phase))
@@ -83,14 +84,14 @@ int SlicerLayer::tryFaceNextSegmentIdx(const SlicerSegment& segment, int face_id
     return -1;
 }
 
-int SlicerLayer::getNextSegmentIdx(const SlicerSegment& segment, unsigned int start_segment_idx)
+int SlicerLayer::getNextSegmentIdx(const SlicerSegment& segment, const size_t start_segment_idx) const
 {
     int next_segment_idx = -1;
 
-    bool segment_ended_at_edge = segment.endVertex == nullptr;
+    const bool segment_ended_at_edge = segment.endVertex == nullptr;
     if (segment_ended_at_edge)
     {
-        int face_to_try = segment.endOtherFaceIdx;
+        const int face_to_try = segment.endOtherFaceIdx;
         if (face_to_try == -1)
         {
             return -1;
@@ -104,7 +105,7 @@ int SlicerLayer::getNextSegmentIdx(const SlicerSegment& segment, unsigned int st
         const std::vector<uint32_t> &faces_to_try = segment.endVertex->connected_faces;
         for (int face_to_try : faces_to_try)
         {
-            int result_segment_idx =
+            const int result_segment_idx =
                 tryFaceNextSegmentIdx(segment, face_to_try, start_segment_idx);
             if (result_segment_idx == static_cast<int>(start_segment_idx))
             {
@@ -123,10 +124,10 @@ int SlicerLayer::getNextSegmentIdx(const SlicerSegment& segment, unsigned int st
 
 void SlicerLayer::connectOpenPolylines(Polygons& open_polylines)
 {
-    bool allow_reverse = false;
+    constexpr bool allow_reverse = false;
     // Search a bit fewer cells but at cost of covering more area.
     // Since acceptance area is small to start with, the extra is unlikely to hurt much.
-    coord_t cell_size = largest_neglected_gap_first_phase * 2;
+    constexpr coord_t cell_size = largest_neglected_gap_first_phase * 2;
     connectOpenPolylinesImpl(open_polylines, largest_neglected_gap_second_phase, cell_size, allow_reverse);
 }
 
@@ -787,72 +788,61 @@ void SlicerLayer::makePolygons(const Mesh* mesh)
     polygons.removeDegenerateVerts(); // remove verts connected to overlapping line segments
 }
 
-Slicer::Slicer(Mesh* mesh, const coord_t thickness, const size_t slice_layer_count, bool use_variable_layer_heights, std::vector<AdaptiveLayer>* adaptive_layers)
-: mesh(mesh)
+Slicer::Slicer(Mesh* i_mesh, const coord_t thickness, const size_t slice_layer_count, 
+               bool use_variable_layer_heights, std::vector<AdaptiveLayer>* adaptive_layers)
+    : mesh(i_mesh)
 {
     const SlicingTolerance slicing_tolerance = mesh->settings.get<SlicingTolerance>("slicing_tolerance");
+    const coord_t initial_layer_thickness = 
+        Application::getInstance().current_slice->scene.current_mesh_group->settings.get<coord_t>("layer_height_0");
 
     assert(slice_layer_count > 0);
 
     TimeKeeper slice_timer;
 
-    layers.resize(slice_layer_count);
+    layers = 
+        buildLayersWithHeight(slice_layer_count, slicing_tolerance, initial_layer_thickness, thickness, 
+            use_variable_layer_heights, adaptive_layers);
 
-    // set (and initialize compensation for) initial layer, depending on slicing mode
-    const coord_t initial_layer_thickness = Application::getInstance().current_slice->scene.current_mesh_group->settings.get<coord_t>("layer_height_0");
-    layers[0].z = std::max(0LL, initial_layer_thickness - thickness);
-    coord_t adjusted_layer_offset = initial_layer_thickness;
-    if (use_variable_layer_heights)
-    {
-        layers[0].z = adaptive_layers->at(0).z_position;
-    }
-    else if (slicing_tolerance == SlicingTolerance::MIDDLE)
-    {
-        layers[0].z = initial_layer_thickness / 2;
-        adjusted_layer_offset = initial_layer_thickness + (thickness / 2);
-    }
 
-    // define all layer z positions (depending on slicing mode, see above)
-    for (unsigned int layer_nr = 1; layer_nr < slice_layer_count; layer_nr++)
+    std::vector<std::pair<int32_t, int32_t>> zbbox = buildZHeightsForFaces(*mesh);
+
+
+    buildSegments(*mesh, zbbox, layers);
+
+    log("slice of mesh took %.3f seconds\n", slice_timer.restart());
+
+    makePolygons(*i_mesh, slicing_tolerance, layers);
+    log("slice make polygons took %.3f seconds\n", slice_timer.restart());
+}
+
+void Slicer::buildSegments(const Mesh& mesh, const std::vector<std::pair<int32_t, int32_t>> &zbbox,
+    std::vector<SlicerLayer>& layers) 
+{
+    // OpenMP
+#pragma omp parallel for default(none) shared(mesh, zbbox, layers)
+    // Use a signed type for the loop counter so MSVC compiles (because it uses OpenMP 2.0, an old version).
+    for (int layer_nr = 0; layer_nr < static_cast<int>(layers.size()); layer_nr++)
     {
-        if (use_variable_layer_heights)
+        int32_t z = layers[layer_nr].z;
+        layers[layer_nr].segments.reserve(100);
+
+        // loop over all mesh faces
+        for (unsigned int mesh_idx = 0; mesh_idx < mesh.faces.size(); mesh_idx++)
         {
-            layers[layer_nr].z = adaptive_layers->at(layer_nr).z_position;
-        }
-        else
-        {
-            layers[layer_nr].z = adjusted_layer_offset + (thickness * (layer_nr - 1));
-        }
-    }
+            if ((z < zbbox[mesh_idx].first) || (z > zbbox[mesh_idx].second))
+                 continue;
 
-    // loop over all mesh faces
-    for (unsigned int mesh_idx = 0; mesh_idx < mesh->faces.size(); mesh_idx++)
-    {
-        // get all vertices per face
-        const MeshFace& face = mesh->faces[mesh_idx];
-        const MeshVertex& v0 = mesh->vertices[face.vertex_index[0]];
-        const MeshVertex& v1 = mesh->vertices[face.vertex_index[1]];
-        const MeshVertex& v2 = mesh->vertices[face.vertex_index[2]];
+            // get all vertices per face
+            const MeshFace& face = mesh.faces[mesh_idx];
+            const MeshVertex& v0 = mesh.vertices[face.vertex_index[0]];
+            const MeshVertex& v1 = mesh.vertices[face.vertex_index[1]];
+            const MeshVertex& v2 = mesh.vertices[face.vertex_index[2]];
 
-        // get all vertices represented as 3D point
-        Point3 p0 = v0.p;
-        Point3 p1 = v1.p;
-        Point3 p2 = v2.p;
-
-        // find the minimum and maximum z point
-        int32_t minZ = p0.z;
-        int32_t maxZ = p0.z;
-        if (p1.z < minZ) minZ = p1.z;
-        if (p2.z < minZ) minZ = p2.z;
-        if (p1.z > maxZ) maxZ = p1.z;
-        if (p2.z > maxZ) maxZ = p2.z;
-
-        // calculate all intersections between a layer plane and a triangle
-        for (unsigned int layer_nr = 0; layer_nr < layers.size(); layer_nr++)
-        {
-            int32_t z = layers.at(layer_nr).z;
-
-            if (z < minZ) continue;
+            // get all vertices represented as 3D point
+            Point3 p0 = v0.p;
+            Point3 p1 = v1.p;
+            Point3 p2 = v2.p;
 
             SlicerSegment s;
             s.endVertex = nullptr;
@@ -915,16 +905,54 @@ Slicer::Slicer(Mesh* mesh, const coord_t thickness, const size_t slice_layer_cou
             layers[layer_nr].segments.push_back(s);
         }
     }
+}
 
-    log("slice of mesh took %.3f seconds\n",slice_timer.restart());
+std::vector<SlicerLayer> Slicer::buildLayersWithHeight(size_t slice_layer_count, SlicingTolerance slicing_tolerance,
+    coord_t initial_layer_thickness, coord_t thickness, bool use_variable_layer_heights,
+    const std::vector<AdaptiveLayer>* adaptive_layers)
+{
+    std::vector<SlicerLayer> layers_res;
 
+    layers_res.resize(slice_layer_count);
+
+    // set (and initialize compensation for) initial layer, depending on slicing mode
+    layers_res[0].z = std::max(0LL, initial_layer_thickness - thickness);
+    coord_t adjusted_layer_offset = initial_layer_thickness;
+    if (use_variable_layer_heights)
+    {
+        layers_res[0].z = (*adaptive_layers)[0].z_position;
+    }
+    else if (slicing_tolerance == SlicingTolerance::MIDDLE)
+    {
+        layers_res[0].z = initial_layer_thickness / 2;
+        adjusted_layer_offset = initial_layer_thickness + (thickness / 2);
+    }
+
+    // define all layer z positions (depending on slicing mode, see above)
+    for (unsigned int layer_nr = 1; layer_nr < slice_layer_count; layer_nr++)
+    {
+        if (use_variable_layer_heights)
+        {
+            layers_res[layer_nr].z = (*adaptive_layers)[layer_nr].z_position;
+        }
+        else
+        {
+            layers_res[layer_nr].z = adjusted_layer_offset + (thickness * (layer_nr - 1));
+        }
+    }
+
+    return layers_res;
+}
+
+void Slicer::makePolygons(Mesh& mesh, SlicingTolerance slicing_tolerance, std::vector<SlicerLayer>& layers)
+{
     std::vector<SlicerLayer>& layers_ref = layers; // force layers not to be copied into the threads
 
 #pragma omp parallel for default(none) shared(mesh, layers_ref)
     // Use a signed type for the loop counter so MSVC compiles (because it uses OpenMP 2.0, an old version).
     for (int layer_nr = 0; layer_nr < static_cast<int>(layers_ref.size()); layer_nr++)
     {
-        layers_ref[layer_nr].makePolygons(mesh);
+        layers_ref[layer_nr].makePolygons(&mesh);
     }
 
     switch(slicing_tolerance)
@@ -948,12 +976,12 @@ Slicer::Slicer(Mesh* mesh, const coord_t thickness, const size_t slice_layer_cou
         ;
     }
 
-    LayerIndex layer_apply_initial_xy_offset = 0;
+   LayerIndex layer_apply_initial_xy_offset = 0;
     if (layers.size() > 0 && layers[0].polygons.size() == 0
-        && !mesh->settings.get<bool>("support_mesh")
-        && !mesh->settings.get<bool>("anti_overhang_mesh")
-        && !mesh->settings.get<bool>("cutting_mesh")
-        && !mesh->settings.get<bool>("infill_mesh"))
+        && !mesh.settings.get<bool>("support_mesh")
+        && !mesh.settings.get<bool>("anti_overhang_mesh")
+        && !mesh.settings.get<bool>("cutting_mesh")
+        && !mesh.settings.get<bool>("infill_mesh"))
     {
         layer_apply_initial_xy_offset = 1;
     }
@@ -962,7 +990,7 @@ Slicer::Slicer(Mesh* mesh, const coord_t thickness, const size_t slice_layer_cou
     // Use a signed type for the loop counter so MSVC compiles (because it uses OpenMP 2.0, an old version).
     for (int layer_nr = 0; layer_nr < static_cast<int>(layers_ref.size()); layer_nr++)
     {
-        const coord_t xy_offset = mesh->settings.get<coord_t>((layer_nr <= layer_apply_initial_xy_offset) ? "xy_offset_layer_0" : "xy_offset");
+        const coord_t xy_offset = mesh.settings.get<coord_t>((layer_nr <= layer_apply_initial_xy_offset) ? "xy_offset_layer_0" : "xy_offset");
 
         if (xy_offset != 0)
         {
@@ -970,19 +998,54 @@ Slicer::Slicer(Mesh* mesh, const coord_t thickness, const size_t slice_layer_cou
         }
     }
 
-    mesh->expandXY(mesh->settings.get<coord_t>("xy_offset"));
-    log("slice make polygons took %.3f seconds\n", slice_timer.restart());
+    mesh.expandXY(mesh.settings.get<coord_t>("xy_offset"));
 }
 
-coord_t Slicer::interpolate(const coord_t x, const coord_t x0, const coord_t x1, const coord_t y0, const coord_t y1) const
+
+std::vector<std::pair<int32_t, int32_t>> Slicer::buildZHeightsForFaces(const Mesh& mesh) 
 {
-    const coord_t dx_01 = x1 - x0;
-    coord_t num = (y1 - y0) * (x - x0);
-    num += num > 0 ? dx_01 / 2 : -dx_01 / 2; // add in offset to round result
-    return y0 + num / dx_01;
+    std::vector<std::pair<int32_t, int32_t>> zHeights;
+    zHeights.reserve(mesh.faces.size());
+    for(const auto& face : mesh.faces)
+    {
+        //const MeshFace& face = mesh.faces[mesh_idx];
+        const MeshVertex& v0 = mesh.vertices[face.vertex_index[0]];
+        const MeshVertex& v1 = mesh.vertices[face.vertex_index[1]];
+        const MeshVertex& v2 = mesh.vertices[face.vertex_index[2]];
+
+        // get all vertices represented as 3D point
+        Point3 p0 = v0.p;
+        Point3 p1 = v1.p;
+        Point3 p2 = v2.p;
+
+        // find the minimum and maximum z point		
+        int32_t minZ = p0.z;
+        if (p1.z < minZ) 
+        {
+            minZ = p1.z;
+        }
+        if (p2.z < minZ) 
+        {
+            minZ = p2.z;
+        }
+
+        int32_t maxZ = p0.z;
+        if (p1.z > maxZ) 
+        {
+            maxZ = p1.z;
+        }
+        if (p2.z > maxZ) 
+        {
+            maxZ = p2.z;
+        }
+
+        zHeights.emplace_back(std::make_pair(minZ, maxZ));
+    }
+
+    return zHeights;
 }
 
-SlicerSegment Slicer::project2D(const Point3& p0, const Point3& p1, const Point3& p2, const coord_t z) const
+SlicerSegment Slicer::project2D(const Point3& p0, const Point3& p1, const Point3& p2, const coord_t z)
 {
     SlicerSegment seg;
 
@@ -994,4 +1057,13 @@ SlicerSegment Slicer::project2D(const Point3& p0, const Point3& p1, const Point3
     return seg;
 }
 
+coord_t Slicer::interpolate(const coord_t x, const coord_t x0, const coord_t x1, const coord_t y0, const coord_t y1)
+{
+    const coord_t dx_01 = x1 - x0;
+    coord_t num = (y1 - y0) * (x - x0);
+    num += num > 0 ? dx_01 / 2 : -dx_01 / 2; // add in offset to round result
+    return y0 + num / dx_01;
+}
+
+
 }//namespace cura
